@@ -1,8 +1,6 @@
 # Copyright (C) 2011-2012  Patrick Totzke <patricktotzke@gmail.com>
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-from __future__ import absolute_import
-
 import argparse
 import logging
 import os
@@ -18,41 +16,50 @@ from alot.commands import *
 from alot.commands import CommandParseError, COMMANDS
 from alot.utils import argparse as cargparse
 
+from twisted.internet import asyncioreactor
+asyncioreactor.install()
 
-_SUBCOMMANDS = ['search', 'compose', 'bufferlist', 'taglist', 'pyshell']
+
+_SUBCOMMANDS = ['search', 'compose', 'bufferlist', 'taglist', 'namedqueries',
+                'pyshell']
 
 
 def parser():
     """Parse command line arguments, validate them, and return them."""
-    parser = argparse.ArgumentParser()
-    parser.add_argument('-v', '--version', action='version',
-                        version=alot.__version__)
+    parser = argparse.ArgumentParser(add_help=False)
     parser.add_argument('-r', '--read-only', action='store_true',
-                        help='open db in read only mode')
-    parser.add_argument('-c', '--config',
+                        help='open notmuch database in read-only mode')
+    parser.add_argument('-c', '--config', metavar='FILENAME',
                         action=cargparse.ValidatedStoreAction,
                         validator=cargparse.require_file,
-                        help='config file')
-    parser.add_argument('-n', '--notmuch-config', default=os.environ.get(
+                        help='configuration file')
+    parser.add_argument('-n', '--notmuch-config', metavar='FILENAME',
+                        default=os.environ.get(
                             'NOTMUCH_CONFIG',
                             os.path.expanduser('~/.notmuch-config')),
                         action=cargparse.ValidatedStoreAction,
                         validator=cargparse.require_file,
-                        help='notmuch config')
-    parser.add_argument('-C', '--colour-mode',
-                        choices=(1, 16, 256), type=int, default=256,
-                        help='terminal colour mode [default: %(default)s].')
-    parser.add_argument('-p', '--mailindex-path',
+                        help='notmuch configuration file')
+    parser.add_argument('-C', '--colour-mode', metavar='COLOURS',
+                        choices=(1, 16, 256), type=int,
+                        help='number of colours to use')
+    parser.add_argument('-p', '--mailindex-path', metavar='PATH',
                         action=cargparse.ValidatedStoreAction,
                         validator=cargparse.require_dir,
                         help='path to notmuch index')
-    parser.add_argument('-d', '--debug-level', default='info',
+    parser.add_argument('-d', '--debug-level', metavar='LEVEL', default='info',
                         choices=('debug', 'info', 'warning', 'error'),
-                        help='debug log [default: %(default)s]')
-    parser.add_argument('-l', '--logfile', default='/dev/null',
+                        help='debug level [default: %(default)s]')
+    parser.add_argument('-l', '--logfile', metavar='FILENAME',
+                        default='/dev/null',
                         action=cargparse.ValidatedStoreAction,
                         validator=cargparse.optional_file_like,
-                        help='logfile [default: %(default)s]')
+                        help='log file [default: %(default)s]')
+    parser.add_argument('-h', '--help', action='help',
+                        help='display this help and exit')
+    parser.add_argument('-v', '--version', action='version',
+                        version=alot.__version__,
+                        help='output version information and exit')
     # We will handle the subcommands in a separate run of argparse as argparse
     # does not support optional subcommands until now.
     parser.add_argument('command', nargs=argparse.REMAINDER,
@@ -92,20 +99,17 @@ def main():
                         filemode='w', format=logformat)
 
     # locate alot config files
+    cpath = options.config
     if options.config is None:
         xdg_dir = get_xdg_env('XDG_CONFIG_HOME',
                               os.path.expanduser('~/.config'))
         alotconfig = os.path.join(xdg_dir, 'alot', 'config')
         if os.path.exists(alotconfig):
-            settings.alot_rc_path = alotconfig
-    else:
-        settings.alot_rc_path = options.config
-
-    settings.notmuch_rc_path = options.notmuch_config
+            cpath = alotconfig
 
     try:
-        settings.read_config()
-        settings.read_notmuch_config()
+        settings.read_config(cpath)
+        settings.read_notmuch_config(options.notmuch_config)
     except (ConfigError, OSError, IOError) as e:
         print('Error when parsing a config file. '
               'See log for potential details.')
