@@ -1,8 +1,7 @@
 # Copyright (C) 2011-2012  Patrick Totzke <patricktotzke@gmail.com>
+# Copyright © 2018 Dylan Baker
 # This file is released under the GNU GPL, version 3 or a later revision.
 # For further details see the COPYING file
-from __future__ import absolute_import
-
 from . import Command, registerCommand
 from .globals import SearchCommand
 
@@ -13,7 +12,7 @@ MODE = 'taglist'
 class TaglistSelectCommand(Command):
 
     """search for messages with selected tag"""
-    def apply(self, ui):
+    async def apply(self, ui):
         tagstring = ui.current_buffer.get_selected_tag()
         cmd = SearchCommand(query=['tag:"%s"' % tagstring])
-        ui.apply_command(cmd)
+        await ui.apply_command(cmd)
