@@ -28,8 +28,8 @@ from ..completion.contacts import ContactsCompleter
 from ..completion.path import PathCompleter
 from ..db.utils import decode_header
 from ..db.utils import formataddr
+from ..db.utils import get_body_part
 from ..db.utils import extract_headers
-from ..db.utils import extract_body
 from ..db.utils import clear_my_address
 from ..db.utils import ensure_unique_address
 from ..db.envelope import Envelope
@@ -506,24 +506,36 @@ class EditNewCommand(Command):
     MODE, 'indent', help='change message/reply indentation',
     arguments=[(['indent'], {'action': cargparse.ValidatedStoreAction,
                              'validator': cargparse.is_int_or_pm})])
+@registerCommand(
+    MODE, 'togglemimetree', help='disply mime tree of the message',
+    forced={'mimetree': 'toggle'},
+    arguments=[(['query'], {'help': 'query used to filter messages to affect',
+                            'nargs': '*'})])
+@registerCommand(
+    MODE, 'togglemimepart', help='switch between html and plain text message',
+    forced={'mimepart': 'toggle'},
+    arguments=[(['query'], {'help': 'query used to filter messages to affect',
+                            'nargs': '*'})])
 class ChangeDisplaymodeCommand(Command):
 
     """fold or unfold messages"""
     repeatable = True
 
     def __init__(self, query=None, visible=None, raw=None, all_headers=None,
-                 indent=None, **kwargs):
+                 indent=None, mimetree=None, mimepart=False, **kwargs):
         """
         :param query: notmuch query string used to filter messages to affect
         :type query: str
         :param visible: unfold if `True`, fold if `False`, ignore if `None`
         :type visible: True, False, 'toggle' or None
-        :param raw: display raw message text.
+        :param raw: display raw message text
         :type raw: True, False, 'toggle' or None
         :param all_headers: show all headers (only visible if not in raw mode)
         :type all_headers: True, False, 'toggle' or None
         :param indent: message/reply indentation
         :type indent: '+', '-', or int
+        :param mimetree: show the mime tree of the message
+        :type mimetree: True, False, 'toggle' or None
         """
         self.query = None
         if query:
@@ -532,8 +544,16 @@ class ChangeDisplaymodeCommand(Command):
         self.raw = raw
         self.all_headers = all_headers
         self.indent = indent
+        self.mimetree = mimetree
+        self.mimepart = mimepart
         Command.__init__(self, **kwargs)
 
+    def _matches(self, msgt):
+        if self.query is None or self.query == '*':
+            return True
+        msg = msgt.get_message()
+        return msg.matches(self.query)
+
     def apply(self, ui):
         tbuffer = ui.current_buffer
 
@@ -558,13 +578,6 @@ class ChangeDisplaymodeCommand(Command):
             messagetrees = [tbuffer.get_selected_messagetree()]
         else:
             messagetrees = tbuffer.messagetrees()
-            if self.query != '*':
-
-                def matches(msgt):
-                    msg = msgt.get_message()
-                    return msg.matches(self.query)
-
-                messagetrees = [m for m in messagetrees if matches(m)]
 
         for mt in messagetrees:
             # determine new display values for this message
@@ -572,11 +585,28 @@ class ChangeDisplaymodeCommand(Command):
                 visible = mt.is_collapsed(mt.root)
             else:
                 visible = self.visible
+            if not self._matches(mt):
+                visible = not visible
+
             if self.raw == 'toggle':
                 tbuffer.focus_selected_message()
             raw = not mt.display_source if self.raw == 'toggle' else self.raw
             all_headers = not mt.display_all_headers \
                 if self.all_headers == 'toggle' else self.all_headers
+            if self.mimepart:
+                if self.mimepart == 'toggle':
+                    message = mt.get_message()
+                    mimetype = {'plain': 'html', 'html': 'plain'}[
+                        message.get_mime_part().get_content_subtype()]
+                    mimepart = get_body_part(message.get_email(), mimetype)
+                elif self.mimepart is True:
+                    mimepart = ui.get_deep_focus().mimepart
+                mt.set_mimepart(mimepart)
+                ui.update()
+            if self.mimetree == 'toggle':
+                tbuffer.focus_selected_message()
+            mimetree = not mt.display_mimetree \
+                if self.mimetree == 'toggle' else self.mimetree
 
             # collapse/expand depending on new 'visible' value
             if visible is False:
@@ -589,6 +619,8 @@ class ChangeDisplaymodeCommand(Command):
                 mt.display_source = raw
             if all_headers is not None:
                 mt.display_all_headers = all_headers
+            if mimetree is not None:
+                mt.display_mimetree = mimetree
             mt.debug()
             # let the messagetree reassemble itself
             mt.reassemble()
@@ -707,7 +739,7 @@ class PipeCommand(Command):
                     pipestrings.append(mail.as_string())
                 elif self.output_format == 'decoded':
                     headertext = extract_headers(mail)
-                    bodytext = extract_body(mail)
+                    bodytext = msg.get_body_text()
                     msgtext = '%s\n\n%s' % (headertext, bodytext)
                     pipestrings.append(msgtext)
 
@@ -986,6 +1018,7 @@ class MoveFocusCommand(MoveCommand):
 
     def apply(self, ui):
         logging.debug(self.movement)
+        original_focus = ui.get_deep_focus()
         tbuffer = ui.current_buffer
         if self.movement == 'parent':
             tbuffer.focus_parent()
@@ -1016,18 +1049,28 @@ class MoveFocusCommand(MoveCommand):
         # TODO add 'next matching' if threadbuffer stores the original query
         # TODO: add next by date..
 
+        if original_focus != ui.get_deep_focus():
+            ui.update()
+
 
 @registerCommand(MODE, 'select')
 class ThreadSelectCommand(Command):
 
     """select focussed element:
         - if it is a message summary, toggle visibility of the message;
-        - if it is an attachment line, open the attachment"""
+        - if it is an attachment line, open the attachment
+        - if it is a mimepart, toggle visibility of the mimepart"""
     async def apply(self, ui):
         focus = ui.get_deep_focus()
         if isinstance(focus, AttachmentWidget):
             logging.info('open attachment')
             await ui.apply_command(OpenAttachmentCommand(focus.get_attachment()))
+        elif getattr(focus, 'mimepart', False):
+            if isinstance(focus.mimepart, Attachment):
+                await ui.apply_command(OpenAttachmentCommand(focus.mimepart))
+            else:
+                await ui.apply_command(ChangeDisplaymodeCommand(
+                    mimepart=True, mimetree='toggle'))
         else:
             await ui.apply_command(ChangeDisplaymodeCommand(visible='toggle'))
 
@@ -1104,10 +1147,13 @@ class TagCommand(Command):
         tbuffer = ui.current_buffer
         if self.all:
             messagetrees = tbuffer.messagetrees()
+            testquery = "thread:%s" % \
+                        tbuffer.get_selected_thread().get_thread_id()
         else:
             messagetrees = [tbuffer.get_selected_messagetree()]
+            testquery = "mid:%s" % tbuffer.get_selected_mid()
 
-        def refresh_widgets():
+        def refresh():
             for mt in messagetrees:
                 mt.refresh()
 
@@ -1121,26 +1167,18 @@ class TagCommand(Command):
             tbuffer.refresh()
 
         tags = [t for t in self.tagsstring.split(',') if t]
+
         try:
-            for mt in messagetrees:
-                m = mt.get_message()
-                if self.action == 'add':
-                    m.add_tags(tags, afterwards=refresh_widgets)
-                if self.action == 'set':
-                    m.add_tags(tags, afterwards=refresh_widgets,
-                               remove_rest=True)
-                elif self.action == 'remove':
-                    m.remove_tags(tags, afterwards=refresh_widgets)
-                elif self.action == 'toggle':
-                    to_remove = []
-                    to_add = []
-                    for t in tags:
-                        if t in m.get_tags():
-                            to_remove.append(t)
-                        else:
-                            to_add.append(t)
-                    m.remove_tags(to_remove)
-                    m.add_tags(to_add, afterwards=refresh_widgets)
+            if self.action == 'add':
+                ui.dbman.tag(testquery, tags, remove_rest=False,
+                             afterwards=refresh)
+            if self.action == 'set':
+                ui.dbman.tag(testquery, tags, remove_rest=True,
+                             afterwards=refresh)
+            elif self.action == 'remove':
+                ui.dbman.untag(testquery, tags, afterwards=refresh)
+            elif self.action == 'toggle':
+                ui.dbman.toggle_tags(testquery, tags, afterwards=refresh)
 
         except DatabaseROError:
             ui.notify('index in read-only mode', priority='error')
