@@ -9,7 +9,7 @@ from ..settings.const import settings
 
 class Thread:
     """
-    A wrapper around a notmuch mailthread (:class:`notmuch.database.Thread`)
+    A wrapper around a notmuch mailthread (:class:`notmuch2.Thread`)
     that ensures persistence of the thread: It can be safely read multiple
     times, its manipulation is done via a :class:`alot.db.DBManager` and it can
     directly provide contained messages as :class:`~alot.db.message.Message`.
@@ -20,11 +20,11 @@ class Thread:
         :param dbman: db manager that is used for further lookups
         :type dbman: :class:`~alot.db.DBManager`
         :param thread: the wrapped thread
-        :type thread: :class:`notmuch.database.Thread`
+        :type thread: :class:`notmuch2.Thread`
         """
         self._dbman = dbman
         self._authors = None
-        self._id = thread.get_thread_id()
+        self._id = thread.threadid
         self._messages = {}
         self._tags = set()
 
@@ -33,14 +33,18 @@ class Thread:
     def refresh(self, thread=None):
         """refresh thread metadata from the index"""
         if not thread:
-            thread = self._dbman._get_notmuch_thread(self._id)
+            with self._dbman._with_notmuch_thread(self._id) as thread:
+                self._refresh(thread)
+        else:
+            self._refresh(thread)
 
-        self._total_messages = thread.get_total_messages()
-        self._notmuch_authors_string = thread.get_authors()
+    def _refresh(self, thread):
+        self._total_messages = len(thread)
+        self._notmuch_authors_string = thread.authors
 
         subject_type = settings.get('thread_subject')
         if subject_type == 'notmuch':
-            subject = thread.get_subject()
+            subject = thread.subject
         elif subject_type == 'oldest':
             try:
                 first_msg = list(thread.get_toplevel_messages())[0]
@@ -50,19 +54,19 @@ class Thread:
         self._subject = subject
 
         self._authors = None
-        ts = thread.get_oldest_date()
+        ts = thread.first
 
         try:
             self._oldest_date = datetime.fromtimestamp(ts)
         except ValueError:  # year is out of range
             self._oldest_date = None
         try:
-            timestamp = thread.get_newest_date()
+            timestamp = thread.last
             self._newest_date = datetime.fromtimestamp(timestamp)
         except ValueError:  # year is out of range
             self._newest_date = None
 
-        self._tags = {t for t in thread.get_tags()}
+        self._tags = {t for t in thread.tags}
         self._messages = {}  # this maps messages to its children
         self._toplevel_messages = []
 
@@ -230,21 +234,19 @@ class Thread:
                 :class:`~alot.db.message.Message`.
         """
         if not self._messages:  # if not already cached
-            query = self._dbman.query('thread:' + self._id)
-            thread = next(query.search_threads())
+            with self._dbman._with_notmuch_thread(self._id) as thread:
 
-            def accumulate(acc, msg):
-                M = Message(self._dbman, msg, thread=self)
-                acc[M] = []
-                r = msg.get_replies()
-                if r is not None:
-                    for m in r:
+                def accumulate(acc, msg):
+                    M = Message(self._dbman, msg, thread=self)
+                    acc[M] = []
+                    for m in msg.replies():
                         acc[M].append(accumulate(acc, m))
-                return M
+                    return M
 
-            self._messages = {}
-            for m in thread.get_toplevel_messages():
-                self._toplevel_messages.append(accumulate(self._messages, m))
+                self._messages = {}
+                for m in thread.toplevel():
+                    self._toplevel_messages.append(accumulate(self._messages,
+                                                              m))
         return self._messages
 
     def get_replies_to(self, msg):
