@@ -8,6 +8,7 @@ import email
 import email.charset as charset
 import email.policy
 import email.utils
+from email.errors import MessageError
 import tempfile
 import re
 import logging
@@ -94,7 +95,7 @@ def get_params(mail, failobj=None, header='content-type', unquote=True):
     return {k.lower(): v for k, v in mail.get_params(failobj, header, unquote)}
 
 
-def _handle_signatures(original, message, params):
+def _handle_signatures(original_bytes, original, message, params):
     """Shared code for handling message signatures.
 
     RFC 3156 is quite strict:
@@ -102,6 +103,11 @@ def _handle_signatures(original, message, params):
     * the second is of type 'application/pgp-signature'
     * the second contains the detached signature
 
+    :param original_bytes: the original top-level mail raw bytes,
+        containing the segments against which signatures will be verified.
+        Necessary because parsing and re-serialising a Message isn't
+        byte-perfect, which interferes with signature validation.
+    :type original_bytes: bytes
     :param original: The original top-level mail. This is required to attache
         special headers to
     :type original: :class:`email.message.Message`
@@ -110,33 +116,58 @@ def _handle_signatures(original, message, params):
     :param params: the message parameters as returned by :func:`get_params`
     :type params: dict[str, str]
     """
-    malformed = None
-    if len(message.get_payload()) != 2:
-        malformed = 'expected exactly two messages, got {0}'.format(
-            len(message.get_payload()))
-    else:
-        ct = message.get_payload(1).get_content_type()
+    try:
+        nb_parts = len(message.get_payload()) if message.is_multipart() else 1
+        if nb_parts != 2:
+            raise MessageError(
+                f'expected exactly two messages, got {nb_parts}')
+
+        signature_part = message.get_payload(1)
+        ct = signature_part.get_content_type()
         if ct != _APP_PGP_SIG:
-            malformed = 'expected Content-Type: {0}, got: {1}'.format(
-                _APP_PGP_SIG, ct)
+            raise MessageError(
+               f'expected Content-Type: {_APP_PGP_SIG}, got: {ct}')
 
-    # TODO: RFC 3156 says the alg has to be lower case, but I've seen a message
-    # with 'PGP-'. maybe we should be more permissive here, or maybe not, this
-    # is crypto stuff...
-    if not params.get('micalg', 'nothing').startswith('pgp-'):
-        malformed = 'expected micalg=pgp-..., got: {0}'.format(
-            params.get('micalg', 'nothing'))
+        # TODO[dcbaker]: RFC 3156 says the alg has to be lower case, but I've
+        #  seen a message with 'PGP-'. maybe we should be more permissive here,
+        #  or maybe not, this is crypto stuff...
+        mic_alg = params.get('micalg', 'nothing')
+        if not mic_alg.startswith('pgp-'):
+            raise MessageError(f'expected micalg=pgp-..., got: {mic_alg}')
 
-    sigs = []
-    if not malformed:
-        try:
-            sigs = crypto.verify_detached(
-                message.get_payload(0).as_bytes(policy=email.policy.SMTP),
-                message.get_payload(1).get_payload(decode=True))
-        except GPGProblem as e:
-            malformed = str(e)
+        # RFC 3156 section 5 says that "signed message and transmitted message
+        # MUST be identical". We therefore need to validate against the message
+        # as it was originally sent.
 
-    add_signature_headers(original, sigs, malformed)
+        # The transmitted content and therefore the signed content are using
+        # CRLF as line delimiter, but our eml file has most likely been
+        # converted to UNIX LF line ending in the local storage.
+        if b'\r\n' not in original_bytes:
+            original_bytes = original_bytes.replace(b'\n', b'\r\n')
+
+        # The sender's signed canonical form often differs from the one
+        # produced by Python's standard lib (in the number of blank lines
+        # between multipart segments...). We therefore need to extract the
+        # signed part directly from the original byte string.
+        signed_boundary = b'\r\n--' + message.get_boundary().encode()
+        original_chunks = original_bytes.split(signed_boundary)
+        nb_chunks = len(original_chunks)
+        if nb_chunks != 4:
+            raise MessageError(
+                f'unexpected number of multipart chunks, got {nb_chunks}')
+
+        signed_chunk = original_chunks[1]
+        if len(signed_chunk) < len(b'\r\n'):
+            raise MessageError('signed chunk has an invalid length')
+
+        sigs = crypto.verify_detached(
+            signed_chunk[len(b'\r\n'):],
+            signature_part.get_payload(decode=True))
+
+        add_signature_headers(original, sigs, None)
+
+    except (GPGProblem, MessageError) as error:
+        add_signature_headers(original, [], str(error))
 
 
 def _handle_encrypted(original, message, session_keys=None):
@@ -217,28 +248,16 @@ def _handle_encrypted(original, message, session_keys=None):
         original.attach(content)
 
 
-def decrypted_message_from_file(handle, session_keys=None):
-    '''Reads a mail from the given file-like object and returns an email
-    object, very much like email.message_from_file. In addition to
-    that OpenPGP encrypted data is detected and decrypted. If this
-    succeeds, any mime messages found in the recovered plaintext
-    message are added to the returned message object.
-
-    :param handle: a file-like object
-    :param session_keys: a list OpenPGP session keys
-    :returns: :class:`email.message.Message` possibly augmented with
-              decrypted data
-    '''
-    return decrypted_message_from_message(email.message_from_file(handle,
-                                          _class=email.message.EmailMessage),
-                                          session_keys)
-
-
-def decrypted_message_from_message(m, session_keys=None):
+def _decrypted_message_from_message(original_bytes, m, session_keys=None):
     '''Detect and decrypt OpenPGP encrypted data in an email object. If this
     succeeds, any mime messages found in the recovered plaintext
     message are added to the returned message object.
 
+    :param original_bytes: the original top-level mail raw bytes,
+        containing the segments against which signatures will be verified.
+        Necessary because parsing and re-serialising a Message isn't
+        byte-perfect, which interferes with signature validation.
+    :type original_bytes: bytes
     :param m: an email object
     :param session_keys: a list OpenPGP session keys
     :returns: :class:`email.message.Message` possibly augmented with
@@ -254,7 +273,7 @@ def decrypted_message_from_message(m, session_keys=None):
         # handle OpenPGP signed data
         if (m.get_content_subtype() == 'signed' and
                 p.get('protocol') == _APP_PGP_SIG):
-            _handle_signatures(m, m, p)
+            _handle_signatures(original_bytes, m, m, p)
 
         # handle OpenPGP encrypted data
         elif (m.get_content_subtype() == 'encrypted' and
@@ -272,7 +291,7 @@ def decrypted_message_from_message(m, session_keys=None):
 
                 if (sub.get_content_subtype() == 'signed' and
                         p.get('protocol') == _APP_PGP_SIG):
-                    _handle_signatures(m, sub, p)
+                    _handle_signatures(original_bytes, m, sub, p)
                 elif (sub.get_content_subtype() == 'encrypted' and
                       p.get('protocol') == _APP_PGP_ENC):
                     _handle_encrypted(m, sub, session_keys)
@@ -280,26 +299,14 @@ def decrypted_message_from_message(m, session_keys=None):
     return m
 
 
-def decrypted_message_from_string(s, session_keys=None):
-    '''Reads a mail from the given string. This is the equivalent of
-    :func:`email.message_from_string` which does nothing but to wrap
-    the given string in a StringIO object and to call
-    :func:`email.message_from_file`.
-
-    Please refer to the documentation of :func:`message_from_file` for
-    details.
-
-    '''
-    return decrypted_message_from_file(io.StringIO(s), session_keys)
-
-
 def decrypted_message_from_bytes(bytestring, session_keys=None):
     """Create a Message from bytes.
 
     :param bytes bytestring: an email message as raw bytes
     :param session_keys: a list OpenPGP session keys
     """
-    return decrypted_message_from_message(
+    return _decrypted_message_from_message(
+        bytestring,
         email.message_from_bytes(bytestring,
                                  _class=email.message.EmailMessage,
                                  policy=email.policy.SMTP),
@@ -358,7 +365,7 @@ def render_part(part, field_key='copiousoutput'):
             stdin = raw_payload
 
         # read parameter, create handler command
-        parms = tuple('='.join(p) for p in part.get_params())
+        parms = tuple('='.join(p) for p in part.get_params(failobj=[]))
 
         # create and call external command
         cmd = mailcap.subst(entry['view'], ctype,
@@ -422,11 +429,7 @@ def remove_cte(part, as_string=False):
     # decoding into a str is done at the end if requested
     elif '8bit' in cte:
         logging.debug('assuming Content-Transfer-Encoding: 8bit')
-        # Python's mail library may decode 8bit as raw-unicode-escape, so
-        # we need to encode that back to bytes so we can decode it using
-        # the correct encoding, or it might not, in which case assume that
-        # the str representation we got is correct.
-        bp = payload.encode('raw-unicode-escape')
+        bp = payload.encode('utf8')
 
     elif 'quoted-printable' in cte:
         logging.debug('assuming Content-Transfer-Encoding: quoted-printable')
@@ -459,12 +462,12 @@ def remove_cte(part, as_string=False):
 
 MISSING_HTML_MSG = ("This message contains a text/html part that was not "
                     "rendered due to a missing mailcap entry. "
-                    "Please refer to item 5 in our FAQ: "
+                    "Please refer to item 1 in our FAQ: "
                     "http://alot.rtfd.io/en/latest/faq.html")
 
 
-def extract_body(mail):
-    """Returns a string view of a Message.
+def get_body_part(mail, mimetype=None):
+    """Returns an EmailMessage.
 
     This consults :ref:`prefer_plaintext <prefer-plaintext>`
     to determine if a "text/plain" alternative is preferred over a "text/html"
@@ -476,26 +479,32 @@ def extract_body(mail):
     :rtype: str
     """
 
-    if settings.get('prefer_plaintext'):
-        preferencelist = ('plain', 'html')
-    else:
-        preferencelist = ('html', 'plain')
+    if not mimetype:
+        mimetype = 'plain' if settings.get('prefer_plaintext') else 'html'
+    preferencelist = {
+        'plain': ('plain', 'html'), 'html': ('html', 'plain')}[mimetype]
 
     body_part = mail.get_body(preferencelist)
     if body_part is None:  # if no part matching preferredlist was found
         return ""
 
-    displaystring = ""
+    return body_part
 
-    if body_part.get_content_type() == 'text/plain':
+
+def extract_body_part(body_part):
+    """Returns a string view of a Message."""
+    displaystring = ""
+    rendered_payload = render_part(
+        body_part,
+        **{'field_key': 'view'} if body_part.get_content_type() == 'text/plain'
+        else {})
+    if rendered_payload:  # handler had output
+        displaystring = string_sanitize(rendered_payload)
+    elif body_part.get_content_type() == 'text/plain':
         displaystring = string_sanitize(remove_cte(body_part, as_string=True))
     else:
-        rendered_payload = render_part(body_part)
-        if rendered_payload:  # handler had output
-            displaystring = string_sanitize(rendered_payload)
-        else:
-            if body_part.get_content_type() == 'text/html':
-                displaystring = MISSING_HTML_MSG
+        if body_part.get_content_type() == 'text/html':
+            displaystring = MISSING_HTML_MSG
     return displaystring
 
 
