@@ -27,7 +27,7 @@ DATA_DIRS = get_xdg_env('XDG_DATA_DIRS',
                         '/usr/local/share:/usr/share').split(':')
 
 
-class SettingsManager(object):
+class SettingsManager:
     """Organizes user settings"""
     def __init__(self):
         self.hooks = None
@@ -354,7 +354,7 @@ class SettingsManager(object):
         fallback_focus = resolve_att(onebelow_focus, default_focus)
 
         for sec in cfg['tags'].sections:
-            if re.match('^{}$'.format(re.escape(sec)), tag):
+            if re.match('^{}$'.format(sec), tag):
                 normal = resolve_att(colourpick(cfg['tags'][sec]['normal']),
                                      fallback_normal)
                 focus = resolve_att(colourpick(cfg['tags'][sec]['focus']),
@@ -381,7 +381,7 @@ class SettingsManager(object):
             return getattr(self.hooks, key, None)
         return None
 
-    def get_mapped_input_keysequences(self, mode='global', prefix=u''):
+    def get_mapped_input_keysequences(self, mode='global', prefix=''):
         # get all bindings in this mode
         globalmaps, modemaps = self.get_keybindings(mode)
         candidates = list(globalmaps.keys()) + list(modemaps.keys())
