@@ -17,8 +17,6 @@
 
 """Custom extensions of the argparse module."""
 
-from __future__ import absolute_import
-
 import argparse
 import collections
 import functools
@@ -52,7 +50,7 @@ def _path_factory(check):
 
     @functools.wraps(check)
     def validator(paths):
-        if isinstance(paths, basestring):
+        if isinstance(paths, str):
             check(paths)
         elif isinstance(paths, collections.Sequence):
             for path in paths:
