@@ -5,8 +5,6 @@
 """
 This contains alot-specific :class:`urwid.Widget` used in more than one mode.
 """
-from __future__ import absolute_import
-
 import re
 import operator
 import urwid
@@ -21,6 +19,7 @@ class AttachmentWidget(urwid.WidgetWrap):
     """
     one-line summary of an :class:`~alot.db.attachment.Attachment`.
     """
+
     def __init__(self, attachment, selectable=True):
         self._selectable = selectable
         self.attachment = attachment
@@ -54,7 +53,7 @@ class ChoiceWidget(urwid.Text):
         self.separator = separator
 
         items = []
-        for k, v in choices.iteritems():
+        for k, v in choices.items():
             if v == select and select is not None:
                 items += ['[', k, ']:', v]
             else:
@@ -105,9 +104,10 @@ class CompleteEdit(urwid.Edit):
         :ctrl u: deletes everything from the cursor to the beginning of the
                  input
     """
+
     def __init__(self, completer, on_exit,
                  on_error=None,
-                 edit_text=u'',
+                 edit_text='',
                  history=None,
                  **kwargs):
         """
@@ -130,7 +130,7 @@ class CompleteEdit(urwid.Edit):
         self.historypos = None
         self.focus_in_clist = 0
 
-        if not isinstance(edit_text, unicode):
+        if not isinstance(edit_text, str):
             edit_text = string_decode(edit_text)
         self.start_completion_pos = len(edit_text)
         self.completions = None
@@ -167,10 +167,10 @@ class CompleteEdit(urwid.Edit):
                 if self.historypos is None:
                     self.history.append(self.edit_text)
                     self.historypos = len(self.history) - 1
-                if key == 'cursor up':
-                    self.historypos = (self.historypos + 1) % len(self.history)
-                else:
+                if key == 'up':
                     self.historypos = (self.historypos - 1) % len(self.history)
+                else:
+                    self.historypos = (self.historypos + 1) % len(self.history)
                 self.set_edit_text(self.history[self.historypos])
         elif key == 'enter':
             self.on_exit(self.edit_text)
@@ -234,6 +234,7 @@ class CompleteEdit(urwid.Edit):
 
 class HeadersList(urwid.WidgetWrap):
     """ renders a pile of header values as key/value list """
+
     def __init__(self, headerslist, key_attr, value_attr, gaps_attr=None):
         """
         :param headerslist: list of key/value pairs to display
@@ -285,6 +286,7 @@ class TagWidget(urwid.AttrMap):
         :attr tag: the notmuch tag
         :type tag: str
     """
+
     def __init__(self, tag, fallback_normal=None, fallback_focus=None):
         self.tag = tag
         representation = settings.get_tagstring_representation(tag,
