@@ -4,18 +4,22 @@
 """
 Widgets specific to thread mode
 """
+import email
 import logging
 import urwid
 
-from urwidtrees import Tree, SimpleTree, CollapsibleTree
+from urwidtrees import Tree, SimpleTree, CollapsibleTree, ArrowTree
 
 from .ansi import ANSIText
 from .globals import TagWidget
 from .globals import AttachmentWidget
 from ..settings.const import settings
+from ..db.attachment import Attachment
 from ..db.utils import decode_header, X_SIGNATURE_MESSAGE_HEADER
 from ..helper import string_sanitize
 
+ANSI_BACKGROUND = settings.get("interpret_ansi_background")
+
 
 class MessageSummaryWidget(urwid.WidgetWrap):
     """
@@ -56,6 +60,19 @@ class MessageSummaryWidget(urwid.WidgetWrap):
                 cols.append(('fixed', tag_widget.width(), tag_widget))
         line = urwid.AttrMap(urwid.Columns(cols, dividechars=1), attr,
                              focus_att)
+
+        # In case when an e-mail has multiple tags, Urwid assumes that only
+        # one of the summary line's TagWidget-s can be focused. Which means
+        # that when the summary line is focused, only one TagWidget is rendered
+        # as focused and the rest of them as unfocused. This forces to render
+        # all TagWidget-s as focused. Issue #1433
+        def _render_wrap(size, focus=False):
+            for c in cols:
+                if isinstance(c, tuple):
+                    c[2].set_map('focus' if focus else 'normal')
+            return urwid.AttrMap.render(line, size, focus)
+        line.render = _render_wrap
+
         urwid.WidgetWrap.__init__(self, line)
 
     def __str__(self):
@@ -80,17 +97,16 @@ class TextlinesList(SimpleTree):
         for each line in content.
         """
         structure = []
-        ansi_background = settings.get("interpret_ansi_background")
 
         # depending on this config setting, we either add individual lines
         # or the complete context as focusable objects.
         if settings.get('thread_focus_linewise'):
             for line in content.splitlines():
                 structure.append((ANSIText(line, attr, attr_focus,
-                                           ansi_background), None))
+                                           ANSI_BACKGROUND), None))
         else:
             structure.append((ANSIText(content, attr, attr_focus,
-                                       ansi_background), None))
+                                       ANSI_BACKGROUND), None))
         SimpleTree.__init__(self, structure)
 
 
@@ -160,8 +176,10 @@ class MessageTree(CollapsibleTree):
         self._all_headers_tree = None
         self._default_headers_tree = None
         self.display_attachments = True
+        self._mimetree = None
         self._attachments = None
         self._maintree = SimpleTree(self._assemble_structure(True))
+        self.display_mimetree = False
         CollapsibleTree.__init__(self, self._maintree)
 
     def get_message(self):
@@ -179,6 +197,7 @@ class MessageTree(CollapsibleTree):
         logging.debug('display_source %s', self.display_source)
         logging.debug('display_all_headers %s', self.display_all_headers)
         logging.debug('display_attachements %s', self.display_attachments)
+        logging.debug('display_mimetree %s', self.display_mimetree)
         logging.debug('AHT %s', str(self._all_headers_tree))
         logging.debug('DHT %s', str(self._default_headers_tree))
         logging.debug('MAINTREE %s', str(self._maintree._treelist))
@@ -203,6 +222,9 @@ class MessageTree(CollapsibleTree):
         mainstruct = []
         if self.display_source:
             mainstruct.append((self._get_source(), None))
+        elif self.display_mimetree:
+            mainstruct.append((self._get_headers(), None))
+            mainstruct.append((self._get_mimetree(), None))
         else:
             mainstruct.append((self._get_headers(), None))
 
@@ -316,6 +338,32 @@ class MessageTree(CollapsibleTree):
         gaps_att = settings.get_theming_attribute('thread', 'header')
         return DictList(lines, key_att, value_att, gaps_att)
 
+    def _get_mimetree(self):
+        if self._mimetree is None:
+            tree = self._message.get_mime_tree()
+            tree = self._text_tree_to_widget_tree(tree)
+            tree = SimpleTree([tree])
+            self._mimetree = ArrowTree(tree)
+        return self._mimetree
+
+    def _text_tree_to_widget_tree(self, tree):
+        att = settings.get_theming_attribute('thread', 'body')
+        att_focus = settings.get_theming_attribute('thread', 'body_focus')
+        mimepart = tree[1] if isinstance(
+            tree[1], (email.message.EmailMessage, Attachment)) else None
+        label, subtrees = tree
+        label = ANSIText(
+            label, att, att_focus, ANSI_BACKGROUND, mimepart=mimepart)
+        if subtrees is None or mimepart:
+            return label, None
+        else:
+            return label, [self._text_tree_to_widget_tree(s) for s in subtrees]
+
+    def set_mimepart(self, mimepart):
+        """ Set message widget mime part and invalidate body tree."""
+        self.get_message().set_mime_part(mimepart)
+        self._bodytree = None
+
 
 class ThreadTree(Tree):
     """
